
# q-expansion of newform 180.4.a.c, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 180
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-5], [32], [-36], [-10], [78], [140], [192], [-6], [-16], [-34], [390], [-52], [-408], [114], [-516], [-58], [-892], [120], [-646], [-1168], [732], [1590], [194], [-798], [272], [-156], [1622], [-1074], [-1528], [-2412], [-2106], [-556], [2418], [2840], [2054], [-460], [-2016], [618], [2964], [-370], [-1104], [-2398], [-1278], [4472], [1340], [2360], [-1380], [1694], [5190], [-2352], [-3502], [-4788], [-1506], [432], [-54], [-6496], [-466], [4854], [-4516], [-8574], [3476], [-2424], [-1558], [8538], [-988], [2546], [-8556], [-3706], [-11394], [264], [10232], [-562], [-7228], [5736], [9186], [-394], [1614], [1034], [-3708], [-4930], [2592], [2162], [1352], [-5532], [3198], [-1510], [-16086], [5384], [2604], [-11136], [14624], [-11844], [-11284], [-4032], [17562], [-3162], [6764], [17798], [-19996], [-11094], [-900], [-7914], [-2380], [-25726], [-3612], [-2898], [-2664], [-502], [7976], [20414], [6342], [22676], [-7048], [20286], [-16108], [-27456], [12522], [16308], [32078], [4610], [-10782], [-2892], [-29572], [-5766], [3326], [7728], [-21616], [10118], [10460], [-17232], [26912], [13838], [17238], [21698], [-18366], [-30316], [-7494], [11526], [-33820], [-19566], [-40096], [-31884], [-24442], [31944], [17486], [-43434], [10820], [-29976], [-40522], [-15570], [-1084], [8208], [34076], [15072], [24392], [-13602], [-47974], [48330], [9324], [14838], [11360], [6972], [41166], [-22464], [-10192], [-322]]
